package java.awt.image;

import java.util.Hashtable;
import java.awt.Image;

public class PixelGrabber implements ImageConsumer
{
    ImageProducer producer;
    int dstX;
    int dstY;
    int dstW;
    int dstH;
    ColorModel imageModel;
    byte[] bytePixels;
    int[] intPixels;
    int dstOff;
    int dstScan;
    private boolean grabbing;
    private int flags;
    private static final int GRABBEDBITS = 48;
    private static final int DONEBITS = 112;
    
    public PixelGrabber(Image image, int n, int n2, int n3, int n4, int[] array, int n5, int n6) {
        this(image.getSource(), n, n2, n3, n4, array, n5, n6);
    }
    
    public PixelGrabber(ImageProducer producer, int dstX, int dstY, int dstW, int dstH, int[] intPixels, int dstOff, int dstScan) {
        this.producer = producer;
        this.dstX = dstX;
        this.dstY = dstY;
        this.dstW = dstW;
        this.dstH = dstH;
        this.dstOff = dstOff;
        this.dstScan = dstScan;
        this.intPixels = intPixels;
        this.imageModel = ColorModel.getRGBdefault();
    }
    
    public PixelGrabber(Image image, int dstX, int dstY, int dstW, int dstH, boolean b) {
        this.producer = image.getSource();
        this.dstX = dstX;
        this.dstY = dstY;
        this.dstW = dstW;
        this.dstH = dstH;
        if (b) {
            this.imageModel = ColorModel.getRGBdefault();
        }
    }
    
    public synchronized void startGrabbing() {
        if ((this.flags & 0x70) != 0x0) {
            return;
        }
        if (!this.grabbing) {
            this.grabbing = true;
            this.flags &= 0xFFFFFF7F;
            this.producer.startProduction(this);
        }
        return;
    }
    
    public synchronized void abortGrabbing() {
        this.imageComplete(4);
    }
    
    public boolean grabPixels() throws InterruptedException {
        return this.grabPixels(0L);
    }
    
    public synchronized boolean grabPixels(long n) throws InterruptedException {
        if ((this.flags & 0x70) != 0x0) {
            return (this.flags & 0x30) != 0x0;
        }
        long n2 = n + System.currentTimeMillis();
        if (!this.grabbing) {
            this.grabbing = true;
            this.flags &= 0xFFFFFF7F;
            this.producer.startProduction(this);
        }
        while (this.grabbing) {
            long timeoutMillis;
            if (n == 0L) {
                timeoutMillis = 0L;
            }
            else {
                timeoutMillis = n2 - System.currentTimeMillis();
                if (timeoutMillis <= 0L) {
                    break;
                }
            }
            this.wait(timeoutMillis);
        }
        return (this.flags & 0x30) != 0x0;
    }
    
    public synchronized int getStatus() {
        return this.flags;
    }
    
    public synchronized int getWidth() {
        return (this.dstW < 0) ? -1 : this.dstW;
    }
    
    public synchronized int getHeight() {
        return (this.dstH < 0) ? -1 : this.dstH;
    }
    
    public synchronized Object getPixels() {
        return (this.bytePixels == null) ? this.intPixels : this.bytePixels;
    }
    
    public synchronized ColorModel getColorModel() {
        return this.imageModel;
    }
    
    @Override
    public void setDimensions(int n, int n2) {
        if (this.dstW < 0) {
            this.dstW = n - this.dstX;
        }
        if (this.dstH < 0) {
            this.dstH = n2 - this.dstY;
        }
        if (this.dstW <= 0 || this.dstH <= 0) {
            this.imageComplete(3);
        }
        else if (this.intPixels == null && this.imageModel == ColorModel.getRGBdefault()) {
            this.intPixels = new int[this.dstW * this.dstH];
            this.dstScan = this.dstW;
            this.dstOff = 0;
        }
        this.flags |= 0x3;
    }
    
    @Override
    public void setHints(int n) {
    }
    
    @Override
    public void setProperties(Hashtable<?, ?> hashtable) {
    }
    
    @Override
    public void setColorModel(ColorModel colorModel) {
    }
    
    private void convertToRGB() {
        int n = this.dstW * this.dstH;
        int[] intPixels = new int[n];
        if (this.bytePixels != null) {
            for (int j = 0; j < n; ++j) {
                intPixels[j] = this.imageModel.getRGB(this.bytePixels[j] & 0xFF);
            }
        }
        else if (this.intPixels != null) {
            for (int i = 0; i < n; ++i) {
                intPixels[i] = this.imageModel.getRGB(this.intPixels[i]);
            }
        }
        this.bytePixels = null;
        this.intPixels = intPixels;
        this.dstScan = this.dstW;
        this.dstOff = 0;
        this.imageModel = ColorModel.getRGBdefault();
    }
    
    @Override
    public void setPixels(int n, int n2, int i, int j, ColorModel imageModel, byte[] array, int n3, int n4) {
        if (n2 < this.dstY) {
            int n5 = this.dstY - n2;
            if (n5 >= j) {
                return;
            }
            n3 += n4 * n5;
            n2 += n5;
            j -= n5;
        }
        if (n2 + j > this.dstY + this.dstH) {
            j = this.dstY + this.dstH - n2;
            if (j <= 0) {
                return;
            }
        }
        if (n < this.dstX) {
            int n6 = this.dstX - n;
            if (n6 >= i) {
                return;
            }
            n3 += n6;
            n += n6;
            i -= n6;
        }
        if (n + i > this.dstX + this.dstW) {
            i = this.dstX + this.dstW - n;
            if (i <= 0) {
                return;
            }
        }
        int n7 = this.dstOff + (n2 - this.dstY) * this.dstScan + (n - this.dstX);
        if (this.intPixels == null) {
            if (this.bytePixels == null) {
                this.bytePixels = new byte[this.dstW * this.dstH];
                this.dstScan = this.dstW;
                this.dstOff = 0;
                this.imageModel = imageModel;
            }
            else if (this.imageModel != imageModel) {
                this.convertToRGB();
            }
            if (this.bytePixels != null) {
                for (; j > 0; n3 += n4, n7 += this.dstScan, --j) {
                    System.arraycopy(array, n3, this.bytePixels, n7, i);
                }
            }
        }
        if (this.intPixels != null) {
            int n8;
            int n9;
            for (n8 = this.dstScan - i, n9 = n4 - i, ; j > 0; n3 += n9, n7 += n8, --j) {
                for (; i > 0; --i) {
                    this.intPixels[n7++] = imageModel.getRGB(array[n3++] & 0xFF);
                }
            }
        }
        this.flags |= 0x8;
    }
    
    @Override
    public void setPixels(int n, int n2, int i, int j, ColorModel imageModel, int[] array, int n3, int n4) {
        if (n2 < this.dstY) {
            int n5 = this.dstY - n2;
            if (n5 >= j) {
                return;
            }
            n3 += n4 * n5;
            n2 += n5;
            j -= n5;
        }
        if (n2 + j > this.dstY + this.dstH) {
            j = this.dstY + this.dstH - n2;
            if (j <= 0) {
                return;
            }
        }
        if (n < this.dstX) {
            int n6 = this.dstX - n;
            if (n6 >= i) {
                return;
            }
            n3 += n6;
            n += n6;
            i -= n6;
        }
        if (n + i > this.dstX + this.dstW) {
            i = this.dstX + this.dstW - n;
            if (i <= 0) {
                return;
            }
        }
        if (this.intPixels == null) {
            if (this.bytePixels == null) {
                this.intPixels = new int[this.dstW * this.dstH];
                this.dstScan = this.dstW;
                this.dstOff = 0;
                this.imageModel = imageModel;
            }
            else {
                this.convertToRGB();
            }
        }
        int n7 = this.dstOff + (n2 - this.dstY) * this.dstScan + (n - this.dstX);
        if (this.imageModel == imageModel) {
            for (; j > 0; n3 += n4, n7 += this.dstScan, --j) {
                System.arraycopy(array, n3, this.intPixels, n7, i);
            }
        }
        else {
            if (this.imageModel != ColorModel.getRGBdefault()) {
                this.convertToRGB();
            }
            int n8;
            int n9;
            for (n8 = this.dstScan - i, n9 = n4 - i, ; j > 0; n3 += n9, n7 += n8, --j) {
                for (; i > 0; --i) {
                    this.intPixels[n7++] = imageModel.getRGB(array[n3++]);
                }
            }
        }
        this.flags |= 0x8;
    }
    
    @Override
    public synchronized void imageComplete(int n) {
        this.grabbing = false;
        switch (n) {
            case 4: {
                this.flags |= 0x80;
                break;
            }
            case 3: {
                this.flags |= 0x20;
                break;
            }
            case 2: {
                this.flags |= 0x10;
                break;
            }
            default: {
                this.flags |= 0xC0;
                break;
            }
        }
        this.producer.removeConsumer(this);
        this.notifyAll();
    }
    
    public synchronized int status() {
        return this.flags;
    }
}
